#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Utilities for processing SASS and images from default and custom palette.
"""

from .__utils import getAllScreensGeometry, getAllScreensWorkspace, getWorkspace, setStylesheetOnQApp, StyleLooper
